from .imports import *
DIRBASES = {
    "temp":{
        "envKey":"VIDEO_TEMP_DIRECTORY",
        "dirbase":"temp"
        },
    "download":{
        "envKey":"VIDEO_DOWNLOADS_DIRECTORY",
        "dirbase":"downloads"
        },
    "info":{
        "envKey":"VIDEO_INFO_DIRECTORY",
        "dirbase":"info_registry"
        },
    "data":{
        "envKey":"VIDEO_DATA_DIRECTORY",
        "dirbase":"data"
        },
    "video":{
        "envKey":"VIDEO_DIRECTORY",
        "dirname":os.path.expanduser("~"),
        "dirbase":"videos",
        "directory":os.path.join(os.path.expanduser("~"), "videos")
        }
    }
def get_dirbase_data(key=None,directory=None,envKey=None):
    dirbase_data = DIRBASES.get(key,{})
    key = key or dirbase_data.get("dirbase")
    envKey = envKey or dirbase_data.get("envKey")
    directory = directory or dirbase_data.get("directory")
    return directory,key,envKey
def derive_spec_directory(spec_dir,video_directory=None):
    """Default downloads under <video_directory>/videos."""
    video_directory = video_directory or derive_video_directory()
    return os.path.join(video_directory, spec_dir)
def get_spec_directory(directory=None, key=None,envKey=None, envPath=None):
    """Get or create the video download directory."""
    directory = (
        directory
        or get_env_value(key=envKey, path=envPath)
        or derive_spec_directory(key)
    )
    os.makedirs(directory, exist_ok=True)
    return directory



# 🔹 NEW: for downloads
def derive_download_directory(key=None,video_directory=None):
    """Default downloads under <video_directory>/videos."""
    directory,spec_dir,envKey = get_dirbase_data(key='download',directory=video_directory)
    return derive_spec_directory(
        spec_dir=spec_dir,
        video_directory=directory,
        )
def derive_data_directory(key=None,video_directory=None):
    """Default downloads under <video_directory>/videos."""
    directory,spec_dir,envKey = get_dirbase_data(key='data',directory=video_directory)
    return derive_spec_directory(
        spec_dir=spec_dir,
        video_directory=directory,
        )

def derive_temp_directory(key=None,video_directory=None):
    """Default downloads under <video_directory>/videos."""
    directory,spec_dir,envKey = get_dirbase_data(key='temp',directory=video_directory)
    return derive_spec_directory(
        spec_dir=spec_dir,
        video_directory=directory,
        )

def derive_info_registry(key=None,video_directory=None):
    """Default downloads under <video_directory>/videos."""
    directory,spec_dir,envKey = get_dirbase_data(key='info',directory=video_directory)
    return derive_spec_directory(
        spec_dir=spec_dir,
        video_directory=directory,
        )
def derive_video_directory(key=None,video_directory=None):
    """Default downloads under <video_directory>/videos."""
    directory,spec_dir,envKey = get_dirbase_data(key='video',directory=video_directory)
    return derive_spec_directory(
        spec_dir=spec_dir,
        video_directory=directory,
        )


def get_video_directory(key=None,video_directory=None,  envPath=None):
    """Get or create the main video directory."""
    key = key or DIRBASES.get("video",{}).get("envKey")
    video_directory = (
        video_directory
        or get_env_value(key=key, path=envPath)
        or derive_video_directory()
    )
    os.makedirs(video_directory, exist_ok=True)
    return video_directory
def get_download_directory(
    directory=None,
    envKey=None,
    key=None,
    envPath=None,
    **kwargs
    ):
    """Get or create the video download directory."""
    directory,key,envKey = get_dirbase_data(
        key='download',  # ✅ FIXED (singular to match DIRBASES)
        directory=directory or kwargs.get('download_directory'),
        envKey=envKey
    )
    return get_spec_directory(
        directory=directory,
        key=key,
        envKey=envKey,
        envPath=envPath
    )

def get_info_directory(
    directory=None,
    envKey=None,
    key=None,
    envPath=None,
    **kwargs
    ):
    """Get or create the info registry directory."""
    directory,key,envKey = get_dirbase_data(key='info',directory=directory or kwargs.get('info_directory'),envKey=envKey)
    return get_spec_directory(
        directory=directory,
        key=key,
        envKey=envKey,
        envPath=envPath
        )
def get_temp_directory(
    directory=None,
    key=None,
    envKey=None,
    envPath=None,
    **kwargs
    ):
    """Get or create the video download directory."""
    directory,key,envKey = get_dirbase_data(key='temp',directory=directory or kwargs.get('temp_directory'),envKey=envKey)
    return get_spec_directory(
        directory=directory,
        key=key,
        envKey=envKey,
        envPath=envPath
        )
def get_data_directory(
    directory=None,
    key=None,
    envKey=None,
    envPath=None,
    **kwargs
    ):
    """Get or create the video download directory."""
    directory,key,envKey = get_dirbase_data(key='data',directory=directory or kwargs.get('data_directory'),envKey=envKey)
    return get_spec_directory(
        directory=directory,
        key=key,
        envKey=envKey,
        envPath=envPath
        )



from ..imports import *
from .functions import *
class seleneumManager(metaclass=SingletonMeta):
    def __init__(self, url):
        if getattr(self, "initialized", False):
            return
        self.initialized = True

        p = urlparse(url)
        self.domain = p.netloc
        self.scheme = p.scheme or "https"
        self.base_url = f"{self.scheme}://{self.domain}"

        self.site_dir = os.path.join("/var/tmp", "cw-sites", self.domain)
        os.makedirs(self.site_dir, exist_ok=True)

        self._sessions: dict[str, dict] = {}  # key -> {"driver": ..., "profile": ...}
        atexit.register(lambda sm=self: sm.close_all())

    def get_url_to_path(self, url):
        url = eatAll(str(url), ['',' ','\n','\t','\\','/'])
        p = urlparse(url)
        if p.netloc == self.domain:
            parts = [x for x in p.path.split('/') if x]
            d = self.site_dir
            for seg in parts[:-1]:
                d = os.path.join(d, seg)
                os.makedirs(d, exist_ok=True)
            last = parts[-1] if parts else "index.html"
            ext = os.path.splitext(last)[-1] or ".html"
            if not hasattr(self, "page_type"):
                self.page_type = []
            self.page_type.append(ext if not self.page_type else self.page_type[-1])
            return os.path.join(d, last)

    def get_with_netloc(self, url):
        p = urlparse(url)
        if p.netloc == '':
            url = f"{self.scheme}://{self.domain}/{url.strip().lstrip('/')}"
        return url

    def get_driver(self, url) -> tuple[str, webdriver.Chrome]:
        bin_path = get_env_value('CHROME_BINARY')
        opts, prof = _make_chrome_options(binary_path=bin_path, user_data_dir=None)
        driver = webdriver.Chrome(options=opts)
        key = f"{url}#{time.time()}"
        self._sessions[key] = {"driver": driver, "profile": prof}
        return key, driver

    def close_driver(self, key: str):
        sess = self._sessions.pop(key, None)
        if not sess: return
        try:
            try: sess["driver"].quit()
            except Exception: pass
        finally:
            shutil.rmtree(sess.get("profile") or "", ignore_errors=True)

    def close_all(self):
        for key in list(self._sessions.keys()):
            self.close_driver(key)
seleniumManager = seleneumManager



import time,json,hashlib,threading,os,re,yt_dlp,urllib.request,m3u8_To_MP4,subprocess,requests,shutil,tempfile
from abstract_utilities import get_logFile,safe_dump_to_file,get_time_stamp,SingletonMeta
from m3u8 import M3U8  # Install: pip install m3u8
from urllib.parse import urljoin
from yt_dlp.postprocessor.ffmpeg import FFmpegFixupPostProcessor
from abstract_math import divide_it,add_it,multiply_it,subtract_it
from ....soupManager import soupManager
from ....urlManager import *
from abstract_security import get_env_value
logger = get_logFile('video_bp')



import logging
from ..urlManager import (
    urlManager,
    get_url,
    get_url_mgr
    )
from ..requestManager import (
    requestManager,
    get_source,
    get_req_mgr
    )
from ..soupManager import (
    soupManager,
    get_soup,
    get_soup_mgr
    )
from bs4 import BeautifulSoup
logging.basicConfig(level=logging.INFO)



from .functions import *
from .download_utils import *
from .image_utils import *
def optimize_video_for_safari(input_file, reencode=False):
    """
    Optimizes an MP4 file for Safari by moving the 'moov' atom to the beginning.
    Optionally, re-encodes the video for maximum compatibility.
    
    Args:
        input_file (str): Path to the original MP4 file.
        reencode (bool): If True, re-encode the video for Safari compatibility.
        
    Returns:
        str: Path to the optimized MP4 file.
    """
    tmp_dir = tempfile.mkdtemp()
    try:
        local_input = os.path.join(tmp_dir, os.path.basename(input_file))
        shutil.copy2(input_file, local_input)
        
        base, ext = os.path.splitext(local_input)
        local_output = f"{base}_optimized{ext}"
        
        if reencode:
            # Re-encoding command for maximum Safari compatibility
            command = [
                "ffmpeg", "-i", local_input,
                "-c:v", "libx264", "-profile:v", "baseline", "-level", "3.0", "-pix_fmt", "yuv420p",
                "-c:a", "aac", "-b:a", "128k",
                "-movflags", "faststart",
                local_output
            ]
        else:
            # Simple faststart with stream copy
            command = [
                "ffmpeg", "-i", local_input,
                "-c", "copy", "-movflags", "faststart",
                local_output
            ]
        
        try:
            subprocess.run(command, check=True)
            shutil.copy2(local_output, input_file)
            print(f"Optimized video saved as {input_file}")
        except subprocess.CalledProcessError as e:
            print(f"Error during optimization: {e}")
        return input_file
    finally:
        shutil.rmtree(tmp_dir)




def dl_video(url, download_directory=None, output_filename=None,
             get_info=None, download_video=None, ydl_opts=None):
    mgr = get_video_info(
        url,
        download_directory=download_directory,
        output_filename=output_filename,
        get_info=get_info,
        download_video=download_video,
        ydl_opts=ydl_opts,  # pass through
    )
    return get_video_info_from_mgr(mgr)
def for_dl_video(url,download_directory=None,output_filename=None,get_info=None,download_video=None):
    get_info = bool_or_default(get_info,default=True)
    download_video =bool_or_default(download_video,default=True)
    video_mgr = dl_video(url,download_directory=download_directory,output_filename=output_filename,get_info=get_info,download_video=download_video)
    if get_video_info_from_mgr(video_mgr):
        return video_mgr
    videos = soupManager(url).soup.find_all('video')
    for video in videos:
        src = video.get("src")
        video_mgr = dl_video(src,download_directory=download_directory,output_filename=output_filename,download_video=download_video)
        if get_video_info_from_mgr(video_mgr):
            return video_mgr
def downloadvideo(url, directory=None,output_filename=None, rename_display=None, thumbnails=None, audio=None,safari_optimize=None,download_video=None,*args,**kwargs):
    rename_display = bool_or_default(rename_display)
    thumbnails= bool_or_default(thumbnails)
    audio= bool_or_default(thumbnails,default=False)
    safari_optimize=bool_or_default(thumbnails,default=True)
    download_video =bool_or_default(download_video,default=True)
    output_filename = output_filename or get_temp_file_name(url)
    video_mgr = for_dl_video(url,download_directory=directory,output_filename=output_filename,download_video=download_video)
    info = video_mgr.info
    display_id = get_display_id(info)
    os.makedirs(directory, exist_ok=True)
    video_directory = os.path.join(directory, display_id)
    os.makedirs(video_directory, exist_ok=True)
    info['file_path'] = video_directory
    if info:
        file_path = info.get('file_path')
    if rename_display and file_path:
        # Rename using metadata
        video_id = info.get('id', get_temp_id(url))
        title = output_filename or get_video_title(info)
        safe_title = get_safe_title(title)
        final_filename = output_filename or f"{safe_title}_{video_id}"
        final_filename = f"{final_filename}.mp4"
        new_path = os.path.join(video_directory, final_filename)
        if os.path.exists(info['file_path']):
            os.rename(info['file_path'], new_path)
            info['file_path'] = new_path
        info['file_path'] = new_path
            
            # *** Here we call the optimization function ***
    video_path = info.get('file_path')
    if video_path and video_path.lower().endswith('.mp4') and safari_optimize:
        info['file_path'] = optimize_video_for_safari(video_path,reencode=safari_optimize)
    info_path = os.path.join(video_directory, 'info.json')
    if thumbnails:
        info = get_thumbnails(video_directory, info)
    if audio:
        try:
            info = download_audio(directory, info)
        except:
            info['audio_path'] = None
    info['json_path'] = info_path
    safe_dump_to_file(info, info_path)
    return info




class MySocketClient:
    def __init__(self, ip_address=None, port=None,domain=None):
        self.sock
        self.ip_address= ip_address or None
        self.port = port  or None
        
        self.domain = domain  or None
    def receive_data(self):
        chunks = []
        while True:
            chunk = self.sock.recv(4096)
            if chunk:
                chunks.append(chunk)
            else:
                break
        return b''.join(chunks).decode('utf-8')
    def _parse_socket_response_as_json(self, data, *args, **kwargs):
        return self._parse_json(data[data.find('{'):data.rfind('}') + 1], *args, **kwargs)
    def process_data(self):
        data = self.receive_data()
        return self._parse_socket_response_as_json(data)
    def _parse_json(self,json_string):
        return json.loads(json_string)
    def get_ip(self,domain=None):
        try:
            return self.sock.gethostbyname(domain if domain != None else self.domain)
        except self.sock.gaierror:
            return None
    def grt_host_name(self,ip_address=None):
        return self.sock.gethostbyaddr(ip_address if ip_address != None else self.ip_address)
    def toggle_sock(self):
        if self.sock != None:
            self.sock.close()
        else:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            if host and socket:
                self.sock.connect((host, port))
class MySocketClient():
    _instance = None
    @staticmethod
    def get_instance(ip_address='local_host',port=22,domain="example.com"):
        if MySocketClientSingleton._instance is None:
            MySocketClientSingleton._instance = MySocketClient(ip_address=ip_address,port=port,domain=domain)
        elif MySocketClientSingleton._instance.ip_address != ip_address or MySocketClientSingleton._instance.port != port or UrlManagerSingleton._instance.domain != domain:
            MySocketClientSingleton._instance = MySocketClient(ip_address=ip_address,port=port,domain=domain)
        return MySocketClient



from .functions import *
from ..imports import *
from .info_utils import infoRegistry
import threading,os,re,yt_dlp,urllib.request,m3u8_To_MP4,subprocess
from abstract_utilities import get_logFile,safe_dump_to_file

from m3u8 import M3U8  # Install: pip install m3u8
from urllib.parse import urljoin
from yt_dlp.postprocessor.ffmpeg import FFmpegFixupPostProcessor
from abstract_math import divide_it,add_it,multiply_it,subtract_it
from abstract_pandas import *
import os, shutil, threading, time, yt_dlp
from abstract_utilities import safe_dump_to_file
from .info_utils import infoRegistry, generate_video_id  # from above
from ..imports import logger  # assuming you have a logger

def get_download_directory(download_directory):
    d = download_directory or os.path.expanduser("~/Downloads/videos")
    os.makedirs(d, exist_ok=True)
    return d

def get_corrected_url(url: str | list):
    if isinstance(url, list):
        return url
    return url.strip()

class VideoDownloader:
    def __init__(self, url, download_directory=None, user_agent=None,
                 video_extention="mp4", download_video=True,
                 get_info=False, output_filename=None,
                 ydl_opts=None, registry=None, force_refresh=False):

        self.monitoring = True
        self.pause_event = threading.Event()

        self.url = get_corrected_url(url=url)
        self.video_urls = self.url if isinstance(self.url, list) else [self.url]

        self.registry = registry or infoRegistry()
        self.ydl_opts = ydl_opts or {}
        self.get_download = download_video
        self.get_info = get_info
        self.user_agent = user_agent
        self.video_extention = video_extention
        self.download_directory = get_download_directory(download_directory)
        self.output_filename = output_filename
        self.force_refresh = force_refresh
        self.info = {}

        # start immediately (same behavior as before)
        self._start()

    # ---------- threading ----------

    def _start(self):
        self.download_thread = threading.Thread(target=self._download_entrypoint, name="video-download")
        self.download_thread.daemon = True

        self.monitor_thread = threading.Thread(target=self._monitor, name="video-monitor")
        self.monitor_thread.daemon = True

        self.download_thread.start()
        self.monitor_thread.start()

        # Only join the download (monitor is daemon and will exit)
        self.download_thread.join()

    def stop(self):
        self.monitoring = False
        self.pause_event.set()

    # ---------- monitor ----------

    def _monitor(self, interval=30, max_minutes=15):
        start = time.time()
        while self.monitoring:
            logger.info("Monitoring...")
            if time.time() - start > max_minutes * 60:
                logger.info("Monitor: timeout reached, stopping.")
                break
            self.pause_event.wait(interval)
        logger.info("Monitor: exited.")

    # ---------- yt-dlp options ----------

    def _build_ydl_opts(self, outtmpl, extractor_client=None, force_itag18=False):
        fmt = "bestvideo+bestaudio/best"
        if self.video_extention and self.video_extention != "mp4":
            fmt = f"bestvideo[ext={self.video_extention}]+bestaudio[ext=m4a]/best[ext={self.video_extention}]"
        if force_itag18:
            fmt = "18"

        opts = {
            "quiet": True,
            "noprogress": True,
            "external_downloader": "ffmpeg",
            "outtmpl": outtmpl,
            "format": fmt,
            "merge_output_format": "mp4",  # ensure mp4 merge when needed
            "concurrent_fragment_downloads": 3,
            "ratelimit": 0,
            "retries": 5,
            "fragment_retries": 5,
            "ignoreerrors": False,
        }

        if extractor_client:
            # critical for SABR/NSIG issues
            opts.setdefault("extractor_args", {})
            opts["extractor_args"].setdefault("youtube", {})
            opts["extractor_args"]["youtube"]["player_client"] = [extractor_client]

        if self.user_agent:
            opts["http_headers"] = {"User-Agent": self.user_agent}

        # allow caller to override
        if self.ydl_opts:
            # shallow merge to not lose extractor_args
            for k, v in self.ydl_opts.items():
                if k == "extractor_args":
                    base = opts.setdefault("extractor_args", {})
                    for ek, ev in v.items():
                        if isinstance(ev, dict) and isinstance(base.get(ek), dict):
                            base[ek].update(ev)
                        else:
                            base[ek] = ev
                else:
                    opts[k] = v

        return opts

    # ---------- main download ----------

    def _download_entrypoint(self):
        try:
            info = self._download()
            if info:
                self.info = info
        finally:
            # Always stop the monitor
            self.stop()

    def _download(self):
        for video_url in self.video_urls:
            logger.info(f"[VideoDownloader] Processing: {video_url}")

            # --- Registry shortcut
            cached = self.registry.get_video_info(video_url, force_refresh=self.force_refresh)
            if cached and cached.get("file_path") and os.path.isfile(cached["file_path"]):
                logger.info(f"[VideoDownloader] Found cached file: {cached['file_path']}")
                return cached

            tmp_outtmpl = os.path.join(self.download_directory, "%(id)s.%(ext)s")

            try:
                with yt_dlp.YoutubeDL(self._build_ydl_opts(tmp_outtmpl)) as ydl:
                    info = ydl.extract_info(video_url, download=self.get_download)
                    video_id = info.get("id") or generate_video_id(info.get("title") or "video")

                    # --- Canonical base directory
                    dirbase = os.path.join(self.download_directory, video_id)
                    os.makedirs(dirbase, exist_ok=True)

                    # --- Final video file
                    temp_path = ydl.prepare_filename(info)
                    _, ext = os.path.splitext(temp_path)
                    final_path = os.path.join(dirbase, f"video{ext or '.mp4'}")

                    if temp_path != final_path and os.path.isfile(temp_path):
                        shutil.move(temp_path, final_path)

                    # --- Core info
                    info["file_path"] = final_path
                    info["video_id"] = video_id
                    info_path = os.path.join(dirbase, "info.json")
                    safe_dump_to_file(info, info_path)
                    info["info_path"] = info_path

                    # --- Add placeholders for extras
                    info.update({
                        "audio_path": os.path.join(dirbase, "audio.wav"),
                        "whisper_path": os.path.join(dirbase, "whisper.json"),
                        "srt_path": os.path.join(dirbase, "captions.srt"),
                        "metadata_path": os.path.join(dirbase, "metadata.json"),
                        "thumbnails_dir": os.path.join(dirbase, "thumbnails"),
                    })
                    os.makedirs(info["thumbnails_dir"], exist_ok=True)

                    # --- Update registry
                    self.registry.edit_info(
                        data={"file_path": final_path, "info_path": info_path, **info},
                        url=video_url,
                        video_id=video_id
                    )

                    logger.info(f"[VideoDownloader] Downloaded and stored at {final_path}")
                    return info

            except Exception as e:
                logger.error(f"[VideoDownloader] Download failed: {e}")
                return None




from ..imports import *
def download_image(url, save_path=None):
    """
    Downloads an image from a URL and saves it to the specified path.
    
    Args:
        url (str): The URL of the image to download
        save_path (str, optional): Path to save the image. If None, uses the filename from URL
        
    Returns:
        str: Path where the image was saved, or None if download failed
    """
    try:
        # Send GET request to the URL
        response = requests.get(url, stream=True)
        
        # Check if the request was successful
        if response.status_code == 200:
            # Set decode_content=True to automatically handle Content-Encoding
            response.raw.decode_content = True
            
            # If no save_path provided, extract filename from URL
            if save_path is None:
                # Get filename from URL
                filename = url.split('/')[-1]
                save_path = filename
            
            # Ensure the directory exists
            os.makedirs(os.path.dirname(save_path), exist_ok=True)
            
            # Write the image content to file
            with open(save_path, 'wb') as f:
                f.write(response.content)
            
            print(f"Image successfully downloaded to {save_path}")
            return save_path
        else:
            print(f"Failed to download image. Status code: {response.status_code}")
            return None
            
    except requests.exceptions.RequestException as e:
        print(f"Error downloading image: {str(e)}")
        return None
    except Exception as e:
        print(f"An unexpected error occurred: {str(e)}")
        return None
def get_thumbnails(directory,info):
    thumbnails_dir = os.path.join(directory,'thumbnails')
    os.makedirs(thumbnails_dir, exist_ok=True)
    thumbnails = info.get('thumbnails',[])
    for i,thumbnail_info in enumerate(thumbnails):
        thumbnail_url = thumbnail_info.get('url')
        thumbnail_base_url = thumbnail_url.split('?')[0]
        baseName = os.path.basename(thumbnail_base_url)
        fileName,ext = os.path.splitext(baseName)
        baseName = f"{fileName}{ext}"
        resolution = info['thumbnails'][i].get('resolution')
        if resolution:
            baseName = f"{resolution}_{baseName}"
        img_id = info['thumbnails'][i].get('id')
        if img_id:
            baseName = f"{img_id}_{baseName}"
        thumbnail_path = os.path.join(thumbnails_dir,baseName)
        info['thumbnails'][i]['path']=thumbnail_path
        download_image(thumbnail_url, save_path=thumbnail_path)
    return info



from .functions import *
from .imports import *
def get_registryManager(
    video_directory=None,
    envPath=None,
    info_directory=None
    ):
    return infoRegistry(video_directory=video_directory, envPath=envPath, info_directory=info_directory)
def get_video_info(
    url=None,
    video_id=None,
    force_refresh=False,
    video_directory=None,
    envPath=None,
    info_directory=None,
    video_path=None,
    download=False
    ):
    registryMgr = get_registryManager(video_directory=video_directory, envPath=envPath, info_directory=info_directory)
    return registryMgr.get_video_info(url=url, video_id=video_id, force_refresh=force_refresh,video_path=video_path)
def get_video_info_spec(
    key=None,
    url=None,
    video_id=None,
    force_refresh=False,
    video_directory=None,
    envPath=None,
    info_directory=None,
    video_path=None,
    download=None
    ):

    video_info = get_video_info(
        url=url,
        video_id=video_id,
        force_refresh=force_refresh,
        video_directory=video_directory,
        envPath=envPath,
        info_directory=info_directory,
        video_path=video_path,
        download=download

    )
    if not key:
        return video_info
    value = video_info.get(key)
    if not value:
         value = make_list(get_any_value(video_info,key) or None)[0]
    return value
def get_video_id(
    url=None,
    video_id=None,
    force_refresh=False,
    video_directory=None,
    envPath=None,
    info_directory=None,
    video_path=None,
    download=False
    ):
    if download:
        VideoDownloader(url=url)
    return get_video_info_spec(
        key='id',
        url=url,
        video_id=video_id,
        force_refresh=force_refresh,
        video_directory=video_directory,
        envPath=envPath,
        info_directory=info_directory,
        video_path=video_path,
        download=download,
        
        )
def get_video_title(
    url=None,
    video_id=None,
    force_refresh=False,
    video_directory=None,
    envPath=None,
    info_directory=None,
    video_path=None,
    download=False
    
    ):
    if download:
        VideoDownloader(url=url)
    return get_video_info_spec(
        key='title',
        url=url,
        video_id=video_id,
        force_refresh=force_refresh,
        video_directory=video_directory,
        envPath=envPath,
        info_directory=info_directory,
        video_path=video_path,
        download=download
        )
def get_video_filepath(
    url=None,
    video_id=None,
    force_refresh=False,
    video_directory=None,
    envPath=None,
    info_directory=None,
    video_path=None,
    download=False
    
    ):
    if download:
        VideoDownloader(url=url)
    return get_video_info_spec(
        key='filepath',
        url=url,
        video_id=video_id,
        force_refresh=force_refresh,
        video_directory=video_directory,
        envPath=envPath,
        info_directory=info_directory,
        video_path=video_path,
        download=download
        )


def get_temp_id(url):
    url = str(url)
    url_length = len(url)
    len_neg = 20
    len_neg = len_neg if url_length >= len_neg else url_length
    temp_id = re.sub(r'[^\w\d.-]', '_', url)[-len_neg:]
    return temp_id
def get_temp_file_name(url):
    temp_id = get_temp_id(url)
    temp_filename = f"temp_{temp_id}.mp4"
    return temp_filename
def get_display_id(info):
    display_id = info.get('display_id') or info.get('id')
    return display_id

def get_safe_title(title):
    re_str = r'[^\w\d.-]'
    safe_title = re.sub(re_str, '_', title)
    return safe_title
def get_video_info_from_mgr(video_mgr):
    try:
        info = video_mgr.info
        return info
    except Exception as e:
        print(f"{e}")
        return None



from ..imports import *
from .utils import *
from .utils import _get_video_info
import os, json, time, hashlib, subprocess, unicodedata, re, threading
from datetime import datetime
from abstract_utilities import safe_dump_to_file  # assumes exists

_LOCK = threading.RLock()
def _ensure_standard_paths(info: dict, registry_dir: str,download_directory) -> dict:
    """
    Given a video info dict, populate standard paths for consistency.
    Does not check existence — just defines where things *will* live.
    """
    vid = info.get("video_id") or info.get("id")
    if not vid:
        return info

    dirbase = os.path.join(download_directory, vid)
    os.makedirs(dirbase, exist_ok=True)

    # Always enforce these
    standard = {
        "file_path": info.get("file_path") or os.path.join(dirbase, "video.mp4"),
        "info_path": info.get("info_path") or os.path.join(dirbase, "info.json"),
        "audio_path": os.path.join(dirbase, "audio.wav"),
        "whisper_path": os.path.join(dirbase, "whisper.json"),
        "srt_path": os.path.join(dirbase, "captions.srt"),
        "metadata_path": os.path.join(dirbase, "metadata.json"),
        "thumbnails_dir": os.path.join(dirbase, "thumbnails"),
        "thumbnails_path": os.path.join(dirbase, "thumbnails.json")
    }
    os.makedirs(standard["thumbnails_dir"], exist_ok=True)

    info.update(standard)
    return info
def _sha12(s: str) -> str:
    return hashlib.sha1(s.encode("utf-8")).hexdigest()[:12]

def _atomic_write(path: str, data: dict):
    tmp = f"{path}.tmp"
    safe_dump_to_file(data, tmp)
    os.replace(tmp, path)

def _normalize_ascii(s: str) -> str:
    return unicodedata.normalize('NFKD', s).encode('ascii', 'ignore').decode('ascii')

def generate_video_id(path_or_title: str, max_length: int = 50) -> str:
    base = os.path.splitext(os.path.basename(path_or_title))[0]
    if base == 'video':
        base = os.path.basename(os.path.dirname(path_or_title)) or base
    base = _normalize_ascii(base.lower())
    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
    base = re.sub(r'-{2,}', '-', base)
    if len(base) > max_length:
        h = _sha12(base)
        base = f"{base[:max_length - len(h) - 1].rstrip('-')}-{h}"
    return base or _sha12(path_or_title)

def make_video_info(filepath: str) -> dict:
    import json as _json
    import subprocess as _sub
    cmd = [
        "ffprobe","-v","quiet","-print_format","json",
        "-show_format","-show_streams", filepath
    ]
    probe = _sub.check_output(cmd)
    data = _json.loads(probe)
    info = {
        "id": generate_video_id(filepath),
        "title": os.path.splitext(os.path.basename(filepath))[0],
        "upload_date": datetime.fromtimestamp(os.path.getmtime(filepath)).strftime("%Y%m%d"),
        "duration": float(data["format"].get("duration", 0.0)),
        "streams": data.get("streams", []),
        "format": data.get("format", {}),
        "file_path": os.path.abspath(filepath),
    }
    return info

def _yt_dlp_info(url: str, ydl_opts: dict | None = None) -> dict | None:
    from yt_dlp import YoutubeDL
    opts = {'quiet': True, 'skip_download': True}
    if ydl_opts:
        opts.update(ydl_opts)
    try:
        with YoutubeDL(opts) as ydl:
            return ydl.extract_info(url, download=False)
    except Exception:
        return None

def get_info_directory(directory=None, envPath=None):
    # replace with your own util if you have it already
    d = directory or os.path.expanduser("~/.video_info_cache")
    os.makedirs(d, exist_ok=True)
    return d

class infoRegistry(metaclass=SingletonMeta):
    """Thread-safe registry with consistent paths for each video."""

    def __init__(self, video_directory=None, envPath=None, info_directory=None,
                 temp_directory=None, data_directory=None,download_directory=None, **kwargs):
        if not hasattr(self, 'initialized',):
            self.initialized = True
            self.infoRegistryDirectory = get_info_directory(directory=info_directory, envPath=envPath)
            self.registry_path = os.path.join(self.infoRegistryDirectory, "registry.json")
            self.download_directory = get_download_directory(download_directory)
            self._load_registry()

    def _load_registry(self):
        with _LOCK:
            self.registry = {"by_url": {}, "by_id": {}, "by_path": {}}
            if os.path.isfile(self.registry_path):
                try:
                    with open(self.registry_path, "r", encoding="utf-8") as f:
                        j = json.load(f)
                    self.registry["by_url"].update(j.get("by_url", {}))
                    self.registry["by_id"].update(j.get("by_id", {}))
                    self.registry["by_path"].update(j.get("by_path", {}))
                except Exception:
                    pass

    def _save_registry(self):
        with _LOCK:
            _atomic_write(self.registry_path, self.registry)

    def _get_cache_path(self, video_id: str) -> str:
        return os.path.join(self.infoRegistryDirectory, f"{video_id}.json")

    def _read_cached_info(self, video_id: str) -> dict | None:
        cache = self._get_cache_path(video_id)
        if os.path.isfile(cache):
            try:
                with open(cache, "r", encoding="utf-8") as f:
                    return json.load(f)
            except Exception:
                return None
        return None

    def _write_cached_info(self, video_id: str, info: dict) -> str:
        cache = self._get_cache_path(video_id)
        _atomic_write(cache, info)
        return cache

    def _resolve_video_id(self, url: str | None, video_path: str | None, hint_id: str | None) -> str | None:
        if hint_id:
            return hint_id
        if video_path and video_path in self.registry["by_path"]:
            return self.registry["by_path"][video_path]
        if url and url in self.registry["by_url"]:
            return self.registry["by_url"][url]
        return None

    def _link(self, video_id: str, url: str | None, video_path: str | None):
        with _LOCK:
            if url:
                self.registry["by_url"][url] = video_id
            if video_path:
                self.registry["by_path"][video_path] = video_id
            rec = self.registry["by_id"].get(video_id, {})
            if url:
                rec["url"] = url
            if video_path:
                rec["video_path"] = video_path
            rec["timestamp"] = time.time()
            self.registry["by_id"][video_id] = rec
            self._save_registry()

    def get_video_info(self, url: str | None = None, video_id: str | None = None,
                       force_refresh: bool = False, download: bool = False,
                       video_path: str | None = None) -> dict | None:
        if video_path and os.path.isfile(video_path):
            vid = video_id or generate_video_id(video_path)
            info = make_video_info(video_path)
            cache = self._write_cached_info(vid, info)
            self._link(vid, url, os.path.abspath(video_path))
            info["info_path"] = cache
            info["video_id"] = vid
            return _ensure_standard_paths(info, self.infoRegistryDirectory,self.download_directory)

        if not url and not video_id:
            raise ValueError("Either url or video_id or video_path must be provided")

        vid = self._resolve_video_id(url, video_path, video_id)

        if vid and not force_refresh:
            cached = self._read_cached_info(vid)
            if cached:
                self._link(vid, url, cached.get("file_path"))
                cached["info_path"] = self._get_cache_path(vid)
                cached["video_id"] = vid
                return _ensure_standard_paths(cached, self.infoRegistryDirectory,self.download_directory)

        if url:
            info = _yt_dlp_info(url)
            if info:
                vid = info.get("id") or _sha12(url)
                cache = self._write_cached_info(vid, info)
                self._link(vid, url, None)
                info["info_path"] = cache
                info["video_id"] = vid
                return _ensure_standard_paths(info, self.infoRegistryDirectory,self.download_directory)

        return None

    def edit_info(self, data: dict, url: str | None = None, video_id: str | None = None, video_path: str | None = None):
        cur = self.get_video_info(url=url, video_id=video_id, video_path=video_path, force_refresh=False)
        if not cur:
            raise RuntimeError("No existing info to edit")
        cur.update(data or {})
        vid = cur.get("video_id") or video_id or (url and _sha12(url)) or generate_video_id(video_path or "video")
        cur = _ensure_standard_paths(cur, self.infoRegistryDirectory, self.download_directory)  # ✅ fixed
        cache = self._write_cached_info(vid, cur)
        self._link(vid, url, cur.get("file_path") or video_path)
        cur["info_path"] = cache
        cur["video_id"] = vid
        return cur

    def list_cached_videos(self):
        with _LOCK:
            out = []
            for vid, meta in self.registry["by_id"].items():
                out.append({
                    "video_id": vid,
                    "url": meta.get("url"),
                    "video_path": meta.get("video_path"),
                    "timestamp": meta.get("timestamp"),
                })
            return out



from ..imports import *
def make_video_info(filepath):
    # base ffprobe
    cmd = ["ffprobe","-v","quiet","-print_format","json",
           "-show_format","-show_streams",filepath]
    probe = subprocess.check_output(cmd)
    data = json.loads(probe)

    # derive fields
    info = {
        "id": os.path.basename(filepath).split('.')[0],
        "title": os.path.splitext(os.path.basename(filepath))[0],
        "upload_date": datetime.fromtimestamp(os.path.getmtime(filepath)).strftime("%Y%m%d"),
        "duration": float(data["format"]["duration"]),
        "streams": data["streams"],
        "format": data["format"],
        "video_path": filepath

    }
    return info

def generate_video_id(path: str, max_length: int = 50) -> str:
    # 1. Take basename (no extension)
    file_parts = get_file_parts(path)
    base= file_parts.get("filename")
    if base == 'video':
        base = file_parts.get("dirbase")
    # 2. Normalize Unicode → ASCII
    base = unicodedata.normalize('NFKD', base).encode('ascii', 'ignore').decode('ascii')
    # 3. Lower-case
    base = base.lower()
    # 4. Replace non-alphanumeric with hyphens
    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
    # 5. Collapse duplicates
    base = re.sub(r'-{2,}', '-', base)
    # 6. Optionally truncate & hash for uniqueness
    if len(base) > max_length:
        h = hashlib.sha1(base.encode()).hexdigest()[:8]
        base = base[: max_length - len(h) - 1].rstrip('-') + '-' + h
    return base
def _get_video_info(url=None,file_path=None, ydl_opts=None,output_filename=None, cookies_path=None):
    from yt_dlp import YoutubeDL
    
    
    ydl_opts = {
        'quiet': True,
        'skip_download': True,
    }

    if cookies_path and os.path.exists(cookies_path):
        ydl_opts['cookiefile'] = cookies_path

    try:
        with YoutubeDL(ydl_opts) as ydl:
            info = ydl.extract_info(url, download=False)
            return info
    except Exception as e:
        print(f"Failed to extract video info: {e}")
        return None



# meta_dump.py
from abstract_webtools.managers.networkManager import NetworkManager
from abstract_webtools.managers.userAgentManager import UserAgentManager
from abstract_webtools.managers.soupManager.soupManager import soupManager
import json, sys

def dump_all_meta(url: str):
    ua = UserAgentManager(browser="Chrome", operating_system="Windows")
    net = NetworkManager(user_agent_manager=ua)

    r = net.session.get(url, timeout=30)
    r.raise_for_status()

    sm = soupManager(url=url, source_code=r.text, req_mgr=net)
    out = {
        "url": url,
        "title": sm.soup.title.string.strip() if sm.soup.title and sm.soup.title.string else None,
        "meta": sm.all_meta(),
        "citation": sm.citation_dict(),
        "links": sm.all_links(),
        "json_ld": sm.all_jsonld(),
    }
    print(json.dumps(out, indent=2, ensure_ascii=False))

if __name__ == "__main__":
    url = sys.argv[1]
    dump_all_meta(url)
